/**
 * 
 */
package bloodnamegen;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;

/**
 * @author Neoancient
 * 
 * Copyright Carl Spain
 * 
 *     This file is part of BloodnameGen.
 *
 *  BloodnameGen is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  BloodnameGen is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with BloodnameGen.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

public class MainUI extends JFrame {

	/**
	 * 
	 */
	private static final long serialVersionUID = -8920911331442095338L;
	
	public static final int CBS = 0;
	public static final int CB = 1;
	public static final int CCC = 2;
	public static final int CCO = 3;
	public static final int CDS = 4;
	public static final int CFM = 5;
	public static final int CGB = 6;
	public static final int CGS = 7;
	public static final int CHH = 8;
	public static final int CIH = 9;
	public static final int CJF = 10;
	public static final int CM = 11;
	public static final int CNC = 12;
	public static final int CSJ = 13;
	public static final int CSR = 14;
	public static final int CSA = 15;
	public static final int CSV = 16;
	public static final int CSL = 17;
	public static final int CWI = 18;
	public static final int CW = 19;
	public static final int CWIE = 20;
	public static final int CWOV	 = 21;
	
	public static final Integer[][] terminus = {
		{2807, 3084},
		{2807, 3059},
		{2807, null},
		{2807, null},
		{2807, null},
		{2807, 3073},
		{2807, null},
		{2807, null},
		{2807, null},
		{2807, 3074},
		{2807, null},
		{2807, 2868},
		{2807, null},
		{2807, 3060},
		{2807, null},
		{2807, null},
		{2807, 3075},
		{3075, null},
		{2807, 2834},
		{2807, null},
		{3057, null},
		{2807, 2823}
	};
	
	public static final String[] clans = {
		"CBS", "CB", "CCC", "CCO", "CDS", "CFM", "CGB",
		"CGS", "CHH", "CIH", "CJF", "CM", "CNC", "CSJ",
		"CSR", "CSA", "CSV", "CSL", "CWI", "CW", "CWIE", "CWOV"		
	};
	
	public static final String[] fullNames = {
		"Blood Spirit", "Burrock", "Cloud Cobra", "Coyote",
		"Diamond Shark/Sea Fox", "Fire Mandrill", "Ghost Bear",
		"Goliath Scorpion", "Hell's Horses", "Ice Hellion",
		"Jade Falcon", "Mongoose", "Nova Cat", "Smoke Jaguar",
		"Snow Raven", "Star Adder", "Steel Viper",
		"Stone Lion", "Widowmaker", "Wolf", "Wolf-in-Exile",
		"Wolverine"
	};
	
	public static Integer[] eras = {
		2807, 2825, 2850, 2900, 2950, 3000, 3050, 3060,
		3075, 3085, 3100
	};
	
	private JComboBox<String> cbClan;
	private JComboBox<Integer> cbEra;
	private JComboBox<String> cbPhenotype;
	private JButton btnGo;
	private JLabel lblName;
	private JLabel lblOrigClan;
	private JLabel lblPhenotype;
	private JTextArea txtWarning;

	public MainUI() {
		setTitle ("Bloodname Generator");
 		setDefaultCloseOperation (EXIT_ON_CLOSE);
		
		initUI();
	}
	
	private void initUI() {
		JPanel panMain = new JPanel(new GridBagLayout());
		panMain.setMinimumSize(new java.awt.Dimension(200, 300));
		panMain.setPreferredSize(new java.awt.Dimension(200, 300));
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(5, 5, 5, 5);
		gbc.anchor = GridBagConstraints.NORTHWEST;
		
		cbClan = new JComboBox<String>(fullNames);
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 2;
		panMain.add(cbClan, gbc);
		cbClan.addActionListener(validateActionListener);
		
		cbEra = new JComboBox<Integer>(eras);
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.gridwidth = 2;
		panMain.add(cbEra, gbc);
		cbEra.addActionListener(validateActionListener);
		
		cbPhenotype = new JComboBox<String>();
		cbPhenotype.addItem("None");
		for (int i = 1; i < Bloodname.phenotypeNames.length; i++) {
			cbPhenotype.addItem(Bloodname.phenotypeNames[i]);
		}
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.gridwidth = 2;
		panMain.add(cbPhenotype, gbc);
		cbPhenotype.addActionListener(validateActionListener);
		
		btnGo = new JButton("Go");
		gbc.gridx = 0;
		gbc.gridy = 3;
		gbc.gridwidth = 1;
		panMain.add(btnGo, gbc);
		btnGo.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				Bloodname n = BloodnameSelector.getInstance().randomBloodname
						(clans[cbClan.getSelectedIndex()],
								cbPhenotype.getSelectedIndex(),
								(Integer)cbEra.getSelectedItem());
				lblName.setText(n.getName() + " (" + n.getFounder() + ")");
				lblOrigClan.setText(n.getOrigClan().getFullName((Integer)cbEra.getSelectedItem()));
				lblPhenotype.setText(Bloodname.phenotypeNames[n.getPhenotype()]);
			}
		});
		
		JLabel label = new JLabel("Result:");
		gbc.gridx = 0;
		gbc.gridy = 4;
		gbc.gridwidth = 1;
		panMain.add(label, gbc);

		lblName = new JLabel();
		gbc.gridx = 1;
		gbc.gridy = 4;
		gbc.gridwidth = 1;
		panMain.add(lblName, gbc);

		label = new JLabel("Original Clan:");
		gbc.gridx = 0;
		gbc.gridy = 6;
		gbc.gridwidth = 1;
		panMain.add(label, gbc);

		lblOrigClan = new JLabel();
		gbc.gridx = 1;
		gbc.gridy = 6;
		gbc.gridwidth = 1;
		panMain.add(lblOrigClan, gbc);

		label = new JLabel("Phenotype:");
		gbc.gridx = 0;
		gbc.gridy = 7;
		gbc.gridwidth = 1;
		panMain.add(label, gbc);

		lblPhenotype = new JLabel();
		gbc.gridx = 1;
		gbc.gridy = 7;
		gbc.gridwidth = 1;
		panMain.add(lblPhenotype, gbc);

		txtWarning = new JTextArea();
		txtWarning.setEditable(false);
		txtWarning.setLineWrap(true);
		txtWarning.setWrapStyleWord(true);
		gbc.gridx = 0;
		gbc.gridy = 8;
		gbc.gridwidth = 2;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		panMain.add(txtWarning, gbc);
		
		add(panMain);
 		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
 		setSize(Math.min(350, dim.width), Math.min(300, dim.height));
 		
        int w = getSize().width;
        int h = getSize().height;
        int x = (dim.width - w) / 2;
        int y = (dim.height - h) / 2;

        setLocation(x, y);

	}
	
	private ActionListener validateActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent ev) {
			btnGo.setEnabled(validateInput());
		}
	};
	
	private boolean validateInput() {
		int clan = cbClan.getSelectedIndex();
		int year = (Integer)cbEra.getSelectedItem();
		String phenotype = (String)cbPhenotype.getSelectedItem();
		
		if (clan < 0 || null == phenotype ||
				year < 2807 || year > 3150) {
			return false;
		}
		
		String txt = "";
		
		if (year < terminus[clan][0]) {
			for (int i = 0; i < eras.length; i++) {
				if (eras[i] >= terminus[clan][0]) {
					txt += fullNames[clan]
							+ " formed in "
							+ terminus[clan][0]
							+ ". Using " + eras[i] + ".\n";
					break;
				}
			}
		}
		if (null != terminus[clan][1] &&
				year > terminus[clan][1]) {
			for (int i = eras.length - 1; i >= 0; i--) {
				if (eras[i] <= terminus[clan][1]) {
					txt += fullNames[clan]
						+ " ceased to existed in "
						+ terminus[clan][1]
						+ ". Using " + eras[i] + ".\n";
					break;
				}
			}
		}
		if ("ProtoMech" == phenotype && year < 3060) {
			txt += "ProtoMechs did not exist in "
					+ year + ". Using Aerospace.\n";
		}
		if ("Naval" == phenotype && clan != CSR) {
			txt += "The Naval phenotype is unique to Clan Snow Raven. Using General.\n";
		}
		if ("TankWarrior" == phenotype && clan != CHH) {
			 txt += "The TankWarrior phenotype is unique to Clan Hell's Horses. Using General.\n";
		} else if ("TankWarrior" == phenotype && year < 3100) {
			txt += "The TankWarrior phenotype began development in the 32nd century. Using 3100.\n";
		}
		txtWarning.setText(txt);
		
		return true;
	};
}
