/**
 * 
 */
package bloodnamegen.util;

import java.util.Collection;
import java.util.HashSet;

/**
 * @author Neoancient
 * 
 * A utility class to track fractions without the loss of precision that
 * comes with using floating point numbers.
 *
 * Copyright Carl Spain
 * 
 *     This file is part of BloodnameGen.
 *
 *  BloodnameGen is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  BloodnameGen is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with BloodnameGen.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
public class Fraction {
	
	private int numerator;
	private int denominator;
	
	public Fraction() {
		numerator = 0;
		denominator = 1;
	}
	
	public Fraction(int n, int d) {
		if (d == 0) {
			throw new IllegalArgumentException("Denominator is zero.");
		}
		if (d < 0) {
			n = -n;
			d = -d;
		}
		numerator = n;
		denominator = d;
	}
	
	public Fraction(int i) {
		numerator = i;
		denominator = 1;
	}
	
	public Fraction(Fraction f) {
		numerator = f.numerator;
		denominator = f.denominator;
	}
	
	@Override
	public Object clone() {
		return new Fraction(this);		
	}
	
	@Override
	public String toString() {
		return numerator + "/" + denominator;
	}
	
	public boolean equals(Fraction f) {
		return value() == f.value();
	}
	
	public double value() {
		return (double)numerator / (double)denominator;
	}
	
	public void reduce() {
		if (denominator > 1) {
			for (int i = denominator - 1; i > 1; i--) {
				if (numerator % i == 0 && denominator % i == 0) {
					numerator /= i;
					denominator /= i;
					i = denominator - 1;
				}
			}
		}
	}
	
	public int getNumerator() {
		return numerator;
	}
	
	public int getDenominator() {
		return denominator;
	}
	
	public void add(Fraction f) {
		numerator = numerator * f.denominator + f.numerator * denominator;
		denominator = denominator * f.denominator;
		reduce();
	}
	
	public void add(int i) {
		numerator += i * denominator;
		reduce();
	}

	public void sub(Fraction f) {
		numerator = numerator * f.denominator - f.numerator * denominator;
		denominator = denominator * f.denominator;
		reduce();
	}
	
	public void sub(int i) {
		numerator -= i * denominator;
		reduce();
	}

	public void mul(Fraction f) {
		numerator *= f.numerator;
		denominator *= f.denominator;
		reduce();
	}
	
	public void mul(int i) {
		numerator *= i;
		reduce();
	}

	public void div(Fraction f) {
		numerator *= f.denominator;
		denominator *= f.numerator;
		reduce();
	}
	
	public void div(int i) {
		denominator *= i;
	}

	public static int lcd(Collection<Fraction> list) {
		HashSet<Integer> denominators = new HashSet<Integer>();
		for (Fraction f : list) {
			denominators.add(f.denominator);
		}
		boolean done = false;
		int retVal = 1;
		while (!done) {
			done = true;
			for (Integer d : denominators) {
				if (d / retVal > 1 || retVal % d != 0) {
					retVal++;
					done = false;
					break;
				}
			}
		}
		return retVal;
		
	}
}
